@@ -782,7 +782,7 @@ class test_lsqfit(unittest.TestCase,ArrayTests):
         ([[20.,30.]],[[20.,30.]]),
         ([[20.,30.,40.],[100.,200.,300.]],[[20.,30.]])
         ]:
-            p0 = None if vin is None else dict(s=10.,v=vin)
+            p0 = None if vin is None else dict(s=10., v=vin, dummy=30.)
             p = lsqfit._unpack_p0(p0=p0,p0file=None,prior=prior)
             self.assertEqual(p['s'],0.25 if p0 is None else p0['s'])
             self.assert_arraysequal(p['v'],vout)
@@ -1073,7 +1073,7 @@ class test_lsqfit(unittest.TestCase,ArrayTests):
             '-0.07(20)', '-0.31(20)', '0.12(20)', '0.11(20)', '0.13(20)'
             ])
         prior = gv.BufferDict(a = gv.gvar("0.02(2)"))
-        @transform_p(prior, 0, "p")
+        @transform_p(prior.keys(), 0)
         def fcn(p, N=len(y)):
             "fit function"
             return N * [p['a']]
@@ -1092,15 +1092,36 @@ class test_lsqfit(unittest.TestCase,ArrayTests):
             '-0.07(20)', '-0.31(20)', '0.12(20)', '0.11(20)', '0.13(20)'
             ])
         prior = gv.BufferDict(loga = gv.log(gv.gvar("0.02(2)")))
-        @transform_p(prior, 0, "p")
+        @transform_p(prior.keys(), 0)
         def fcn(p, N=len(y)):
             "fit function"
             return N * [p['a']]
         fit = nonlinear_fit(prior=prior, data=y, fcn=fcn)
         self.assertEqual(gv.exp(fit.p['loga']).fmt(), "0.012(11)")
+        self.assertEqual(fit.transformed_p['a'].fmt(), "0.012(11)")
         self.assertEqual(fcn.__name__, "fcn")
         self.assertEqual(fcn.__doc__, "fit function")
         self.assertTrue(hasattr(fcn, 'transform_p'))
+        with self.assertRaises(IndexError):
+            @transform_p(prior.keys(), 10)
+            def fcn(p):
+                return p
+        with self.assertRaises(ValueError):
+            @transform_p(prior.keys(), pkey='pp')
+            def fcn(p):
+                return p
+        with self.assertRaises(ValueError):
+            @transform_p(prior.keys(),0,'pp')
+            def fcn(p):
+                return p
+        with self.assertRaises(ValueError):
+            @transform_p(prior.keys(),1,'p')
+            def fcn(p):
+                return p
+        with self.assertRaises(ValueError):
+            @transform_p(prior.keys())
+            def fcn(p, otherarg=1):
+                return p
 
     def test_sqrtnormal(self):
         " sqrt-normal priors "
@@ -1111,7 +1132,7 @@ class test_lsqfit(unittest.TestCase,ArrayTests):
             '-0.07(20)', '-0.31(20)', '0.12(20)', '0.11(20)', '0.13(20)'
             ])
         prior = gv.BufferDict(sqrta = gv.sqrt(gv.gvar("0.02(2)")))
-        @transform_p(prior, 1, "p")
+        @transform_p(prior.keys(), 1)
         def fcn(xdummy, p, N=len(y)):
             "fit function"
             return N * [p['a']]
